

/*******************************************************************************
* File: main.c
*
* Description:
*   This code is for PSoC Stick interaction with the Serial API HTML webpage.
*   Adapted from the Big Board demo project but has no LCD to display text, so 
*   just returns the text back to the PC.
* Note:
*  Mini-USB connects to USB port of PC
*
* Hardware Dependency:
*  CY8CKIT-050 PSoC 5LP
*
********************************************************************************
* Copyright 2013, Cypress Semiconductor Corporation. All rights reserved.
* This software is owned by Cypress Semiconductor Corporation and is protected
* by and subject to worldwide patent and copyright laws and treaties.
* Therefore, you may use this software only as provided in the license agreement
* accompanying the software package from which you obtained this software.
* CYPRESS AND ITS SUPPLIERS MAKE NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* WITH REGARD TO THIS SOFTWARE, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT,
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*******************************************************************************/

#include <device.h>
#include <string.h>


# define TEXT "t"
# define LED "l"


uint8 RxDataBuffer[64];                             // 64 byte buffer to hold all data received from webpage
uint8 * pRxDataBuffer = RxDataBuffer;               // Pointer to first byte received
char8 indicator;                                           // Stores indicator t or l
    

void handle(char* buffer, uint16 count) {       /* Handles text and LED values */
    
    indicator = buffer[0];                              // First letter indicates TEXT or LED
        
    if (strcmp(&indicator, TEXT) == 0) {                /* TEXT */
            
        while(USBUART_CDCIsReady() == 0u);              // wait for USBUART to be ready
        
        USBUART_PutData(pRxDataBuffer+1, count-1);      /* Give feedback to HTML Page starting
                                                                  from bytes after indicator      */
        
    } else if (strcmp(&indicator,LED) == 0) {           /* LED */
        
        PWM_WriteCompare(*(pRxDataBuffer+1));           // set Compare Value to byte value after indicator
        
    } 
}

/*******************************************************************************
* Function Name: main
********************************************************************************
*
* Summary:
* Main function performs following functions:
*
* 1: Start the USBUART and PWM
*
* 2: On reset, transmit "*Device Reset*" and turn LED off
*
* 3: Upon receiving packets of data, data is handled depending on the value of the first byte sent (code):
*
*       a. "t" --> indicates that received data is text; return the value to PC
*
*       b. "l" --> indicates that received data is for LED; set PWM compare value
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
int main()
{
    uint16 count;            // count holds number of bytes received (<=64)
    
    CyGlobalIntEnable;
    
    /* Start and initialize the USBUART, and PWM */
   
    USBUART_Start(0u, USBUART_5V_OPERATION); 
    
    while(0 == USBUART_GetConfiguration()) {} 
    
    USBUART_CDC_Init();
  
    PWM_Start();
    
    USBUART_PutString(" *Device reset* \n\r");
    
    PWM_WriteCompare(0);
    
    for(;;)
    {
        if (0u != USBUART_DataIsReady()) {          /* If receiving data */
           
            count = USBUART_GetAll(pRxDataBuffer);  // Put received data (<=64 bytes) into RxDataBuffer 
            
            handle((char*)pRxDataBuffer, count);    // Handle the received data
            
        }
    }
}


/* [] END OF FILE */
